#pragma once

#include "NotifyCommand.h"

///////////////////////////////////////////////////
// CLASS CLoginStateNotifyCommand
///////////////////////////////////////////////////

class CLoginStateNotifyCommand : public INotifyCommand
{
public:
	CLoginStateNotifyCommand(CMessengerActor *pActor, GTMessengerActorState eState) : INotifyCommand(pActor),
		m_pActor(pActor),
		m_eState(eState)
	    { }

	static CLoginStateNotifyCommand* Create(CMessengerActor *pActor, GTMessengerActorState eState)
	{
		return new CLoginStateNotifyCommand(pActor, eState);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();
		for (; it != observers.end(); ++it)
			if((*it)->OnGTMessengerActor_UpdateState(m_pActor, m_eState) == false)
				break;

		//LeaveCriticalSection(pCS);
	}
	
	CMessengerActor *m_pActor;
	GTMessengerActorState m_eState;
};


///////////////////////////////////////////////////
// CLASS CLogoutNotifyCommand
///////////////////////////////////////////////////

class CLogoutNotifyCommand : public INotifyCommand
{
public:
	CLogoutNotifyCommand(CMessengerActor *pActor, GTLogoutReasonType type) : INotifyCommand(pActor),
		m_pActor(pActor),
		m_logoutType(type) { }

	static CLogoutNotifyCommand* Create(CMessengerActor *pActor, GTLogoutReasonType type)
	{
		return new CLogoutNotifyCommand(pActor, type);
	}

	virtual void Prev(void) 
	{
		m_pActor->SetActorState(masLogoutReady, true, true); //  ȣ  Ƽ 
		GT_ASSERT(m_pActor);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);
		m_pActor->SetActorState(masLogoutProgress, false, false); // Ƽ  
		m_pActor->LogoutJob(m_logoutType);
	}

	virtual void Post(void) 
	{
		GT_ASSERT(m_pActor);
		m_pActor->SetActorState(masLogoutOK, true); //  ȣ  Ƽ 
	}

	CMessengerActor *m_pActor;
	GTLogoutReasonType m_logoutType;
};


///////////////////////////////////////////////////
// CLASS CUserStatusNotifyCommand
///////////////////////////////////////////////////

class CUserStatusNotifyCommand : public INotifyCommand
{
public:
	CUserStatusNotifyCommand(CMessengerActor *pActor, BYTE nFlag, BYTE nOnlineStatus, BYTE nPrevIPTStatus, BYTE nCurIPTStatus) : INotifyCommand(pActor),
		m_pActor(pActor),
		m_nFlag(nFlag),
		m_nOnlineStatus(nOnlineStatus),
		m_nPrevIPTStatus(nPrevIPTStatus),
		m_nCurIPTStatus(nCurIPTStatus){ }

	static CUserStatusNotifyCommand* Create(CMessengerActor *pActor, BYTE nFlag, BYTE nOnlineStatus, BYTE nPrevIPTStatus, BYTE nCurIPTStatus)
	{
		return new CUserStatusNotifyCommand(pActor, nFlag, nOnlineStatus, nPrevIPTStatus, nCurIPTStatus);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();
		for (; it != observers.end(); ++it)
			if((*it)->OnGTMessengerActor_UpdateMyStatus(m_pActor, m_nFlag, (GTUserStatusType)m_nOnlineStatus, (NTIPTStatusType)m_nPrevIPTStatus, (NTIPTStatusType)m_nCurIPTStatus) == false)
				break;

		//LeaveCriticalSection(pCS);
	}
	
	CMessengerActor *m_pActor;
	BYTE m_nFlag;
	BYTE m_nOnlineStatus;
	BYTE m_nPrevIPTStatus;
	BYTE m_nCurIPTStatus;
};


///////////////////////////////////////////////////
// CLASS CUserStatusNotifyCommand
///////////////////////////////////////////////////

class CGetBuddyStatusNotifyCommand : public INotifyCommand
{
public:
	CGetBuddyStatusNotifyCommand(CMessengerActor *pActor, std::map<WSTRING, BYTE> &mapBuddyInfo) : INotifyCommand(pActor),
		m_pActor(pActor),
		m_mapBuddyInfo(mapBuddyInfo) { }

	static CGetBuddyStatusNotifyCommand* Create(CMessengerActor *pActor, std::map<WSTRING, BYTE> &mapBuddyInfo)
	{
		return new CGetBuddyStatusNotifyCommand(pActor, mapBuddyInfo);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();
		for (; it != observers.end(); ++it)
			if((*it)->OnGTMessengerActor_GetBuddyStatusNotify(m_pActor, m_mapBuddyInfo) == false)
				break;

		//LeaveCriticalSection(pCS);
	}

	CMessengerActor *m_pActor;
	std::map<WSTRING, BYTE> m_mapBuddyInfo;
};

///////////////////////////////////////////////////
// CLASS CUserNicknameNotifyCommand
///////////////////////////////////////////////////

class CUserTalkNameNotifyCommand : public INotifyCommand
{
public:
	CUserTalkNameNotifyCommand(CMessengerActor *pActor, LPCWSTR strTalkName) : INotifyCommand(pActor),
		m_pActor(pActor),
		m_strTalkName(strTalkName) { }

	static CUserTalkNameNotifyCommand* Create(CMessengerActor *pActor, LPCWSTR strTalkName)
	{
		return new CUserTalkNameNotifyCommand(pActor, strTalkName);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();
		for (; it != observers.end(); ++it)
			if((*it)->OnGTMessengerActor_UpdateMyTalkName(m_pActor, m_strTalkName.c_str()) == false)
				break;

		//LeaveCriticalSection(pCS);
	}
	
	CMessengerActor *m_pActor;
	WSTRING m_strTalkName;
};



///////////////////////////////////////////////////
// CLASS CSystemNotifyCommand
///////////////////////////////////////////////////

class CSystemNotifyCommand : public INotifyCommand
{
public:
	CSystemNotifyCommand(CMessengerActor *pActor, char *pSystemNotice) : INotifyCommand(pActor),
		m_pActor(pActor),
		m_pSystemNotice(pSystemNotice) { }

	static CSystemNotifyCommand* Create(CMessengerActor *pActor, char *pSystemNotice)
	{
		return new CSystemNotifyCommand(pActor, pSystemNotice);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);
		std::string strJsonDoc(m_pSystemNotice);
		Json::Value root;
		Json::Reader reader;
		bool bParsingSuccess = reader.parse(strJsonDoc, root);
		if (!bParsingSuccess)
			return;

		const Json::Value changesNotices = root["changeNoticeList"];
		for ( UINT nIndex = 0; nIndex < changesNotices.size(); ++nIndex ) 
		{
			const Json::Value changeNotice = changesNotices[nIndex]; 
	    	const Json::Value CoListJson = changeNotice["custcoIdList"];
			for(UINT nCoListIndex = 0; nCoListIndex < CoListJson.size(); ++nCoListIndex)
			{
				WSTRING strJsonData = Conversion::Utf8ToWide(CoListJson[nCoListIndex].asString());
				m_ChageNoticeTemp.vecCoList.push_back(strJsonData);
			}
			m_ChageNoticeTemp.strEndYMDT  = Conversion::Utf8ToWide(changeNotice.get("bltnEndYmdtString","").asString());	
			WSTRING strServieID = Conversion::Utf8ToWide(changeNotice.get("serviceId","").asString());	 
			m_ChageNoticeTemp.strExposureHM = Conversion::Utf8ToWide(changeNotice.get("exposureHM","").asString());
			m_ChageNoticeTemp.nChangeNoticeSeq = changeNotice.get("changeNoticeSeq","-1").asInt();
			
			m_ChageNoticeTemp.strLastModifyYMDT = Conversion::Utf8ToWide(changeNotice.get("lastModifyYmdtString","").asString());

			m_ChageNoticeTemp.strChekReason = Conversion::Utf8ToWide(changeNotice.get("changeNotice","").asString());
			m_ChageNoticeTemp.strCheckStartYMDT = Conversion::Utf8ToWide(changeNotice.get("chckStrtYmdtString","").asString());	
			m_ChageNoticeTemp.strChangeTypeCode = Conversion::Utf8ToWide(changeNotice.get("changeTypeCode" , "").asString());	

			const Json::Value CycleCdListJson = changeNotice["exposureCycleCd"];
			for (UINT i=0; i < CycleCdListJson.size(); ++i)
			{
				WSTRING strJsonData =  Conversion::Utf8ToWide(CycleCdListJson[i].asString());
				m_ChageNoticeTemp.vecExposureCycleCd.push_back(strJsonData);
			}	

			const Json::Value LocationListJson = changeNotice["exposureLocation"];
			for (UINT i=0; i < LocationListJson.size(); ++i)
			{
				WSTRING strJsonData =  Conversion::Utf8ToWide(LocationListJson[i].asString());
				m_ChageNoticeTemp.vecExposureLocation.push_back(strJsonData);
			}	
			m_ChageNoticeTemp.strStartYMDT = Conversion::Utf8ToWide(changeNotice.get("bltnStrtYmdtString","").asString());	
			m_ChageNoticeTemp.strCheckEndYMDT = Conversion::Utf8ToWide(changeNotice.get("chckEndYmdtString","").asString());
			if(strServieID.compare(_T("msn_pc")) == 0)
			{
				m_ChageNotice = m_ChageNoticeTemp;
			}


		}
		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();

		for (; it != observers.end(); ++it)
			if((*it)->OnGTMessengerActor_NotifySystemMessage(m_pActor, m_ChageNotice) == false)
				break;

		//LeaveCriticalSection(pCS);
	}
	
	CMessengerActor *m_pActor;

	ChangeNoticePushData m_ChageNoticeTemp;
	ChangeNoticePushData m_ChageNotice;
	char *m_pSystemNotice;
};



///////////////////////////////////////////////////
// CLASS CNoteNotifyCommand
///////////////////////////////////////////////////

class CNoteNotifyCommand : public INotifyCommand
{
public:
	CNoteNotifyCommand(CMessengerActor *pActor, const ULONGLONG &lNoteId, const LPCWSTR strFromID, const LPCWSTR strFromName, const ULONG &nUnreadNoteCnt) : INotifyCommand(pActor),
		m_pActor(pActor),
		m_lNoteId(lNoteId),
		m_strFromID(strFromID),
		m_strFromName(strFromName),
		m_nUnreadNoteCnt(nUnreadNoteCnt) { }

	static CNoteNotifyCommand* Create(CMessengerActor *pActor, const ULONGLONG& lNoteId, const LPCWSTR strFromID, const LPCWSTR strFromName, const ULONG &nUnreadNoteCnt)
	{
		return new CNoteNotifyCommand(pActor, lNoteId, strFromID, strFromName, nUnreadNoteCnt);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();

		for (; it != observers.end(); ++it)
			if((*it)->OnGTMessengerActor_NotifyRecvNote(m_pActor, m_lNoteId, m_strFromID, m_strFromName, m_nUnreadNoteCnt) == false)
				break;

		//LeaveCriticalSection(pCS);
	}
	
	CMessengerActor *m_pActor;

	ULONGLONG m_lNoteId;
	LPCWSTR	m_strFromID;
	LPCWSTR	m_strFromName;
	ULONG	m_nUnreadNoteCnt;
};


///////////////////////////////////////////////////
// CLASS CNoteNotifyUnreadCntCommand
///////////////////////////////////////////////////

class CNoteNotifyUnreadCntCommand : public INotifyCommand
{
public:
	CNoteNotifyUnreadCntCommand(CMessengerActor *pActor, const ULONG& lUnreadCnt) : INotifyCommand(pActor),
		m_pActor(pActor),
		m_lUnreadCnt(lUnreadCnt) { }

	static CNoteNotifyUnreadCntCommand* Create(CMessengerActor *pActor, ULONG& lUnreadCnt)
	{
		return new CNoteNotifyUnreadCntCommand(pActor, lUnreadCnt);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();

		for (; it != observers.end(); ++it)
			if((*it)->OnGTMessengerActor_NotifyRecvUnreadNoteCnt(m_pActor, m_lUnreadCnt) == false)
				break;

		//LeaveCriticalSection(pCS);
	}
	
	CMessengerActor *m_pActor;

	ULONG m_lUnreadCnt;
};

///////////////////////////////////////////////////
// CLASS CMemoNotifyCommand
///////////////////////////////////////////////////

class CMemoNotifyCommand : public INotifyCommand
{
public:
	CMemoNotifyCommand(CMessengerActor *pActor, const P_GT_MEMO_DATA& sNotify) : INotifyCommand(pActor),
		m_pActor(pActor),
		m_sNotify(sNotify) { }

	static CMemoNotifyCommand* Create(CMessengerActor *pActor, const P_GT_MEMO_DATA& sNotify)
	{
		return new CMemoNotifyCommand(pActor, sNotify);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();

		GT_MEMO_DATA sData;
		m_sNotify.Convert(sData);

		for (; it != observers.end(); ++it)
			if((*it)->OnGTMessengerActor_NotifyRecvChatInvite(m_pActor, sData) == false)
				break;

		//LeaveCriticalSection(pCS);
	}
	
	CMessengerActor *m_pActor;

	P_GT_MEMO_DATA m_sNotify;
};



///////////////////////////////////////////////////
// CLASS CSendMemoResultCommand
///////////////////////////////////////////////////

class CSendMemoResultCommand : public INotifyCommand
{
public:
	CSendMemoResultCommand(CMessengerActor *pActor, const P_GT_MEMO_RESULT_DATA& sNotify) : INotifyCommand(pActor),
		m_pActor(pActor),
		m_sNotify(sNotify) { }

	static CSendMemoResultCommand* Create(CMessengerActor *pActor, const P_GT_MEMO_RESULT_DATA& sNotify)
	{
		return new CSendMemoResultCommand(pActor, sNotify);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();

		GT_MEMO_RESULT_DATA sData;
		m_sNotify.Convert(sData);

		for (; it != observers.end(); ++it)
			if((*it)->OnGTMessengerActor_ResponseSendMemo(m_pActor, sData) == false)
				break;

		//LeaveCriticalSection(pCS);
	}
	
	CMessengerActor *m_pActor;

	P_GT_MEMO_RESULT_DATA m_sNotify;
};



///////////////////////////////////////////////////
// CLASS CBuddyStatusNotifyCommand
///////////////////////////////////////////////////

class CBuddyStatusNotifyCommand : public INotifyCommand
{
public:
	CBuddyStatusNotifyCommand(CMessengerActor *pActor, const WSTRING &strBuddyID, int buddyUserUpdateType) : INotifyCommand(pActor),
		m_pActor(pActor),
		m_strBuddyID(strBuddyID),
	    m_buddyUserUpdateType(buddyUserUpdateType) { }

	static CBuddyStatusNotifyCommand* Create(CMessengerActor *pActor, const WSTRING &strBuddyID, int buddyUserUpdateType)
	{
		return new CBuddyStatusNotifyCommand(pActor, strBuddyID, buddyUserUpdateType);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();
		for (; it != observers.end(); ++it)
		{
			(*it)->OnGTMessengerActor_UpdateBuddyStatus(m_pActor, m_strBuddyID.c_str(), m_buddyUserUpdateType); 
		}

		//LeaveCriticalSection(pCS);
	}
	
	CMessengerActor *m_pActor;
	WSTRING m_strBuddyID;
	int m_buddyUserUpdateType;
};


///////////////////////////////////////////////////
// CLASS CNotBuddyUserStatusNotifyCommand
///////////////////////////////////////////////////

class CNotBuddyUserStatusNotifyCommand : public INotifyCommand
{
public:
	CNotBuddyUserStatusNotifyCommand(CMessengerActor *pActor, const WSTRING &strBuddyID, int nUserStatus, const WSTRING &strNick, int nIPTStatus, int nFlag) : INotifyCommand(pActor),
		m_pActor(pActor),
		m_strBuddyID(strBuddyID),
	    m_nUserStatus(nUserStatus),
		m_strNick(strNick),
		m_nIPTStatus(nIPTStatus),
	    m_nFlag(nFlag){ }

	static CNotBuddyUserStatusNotifyCommand* Create(CMessengerActor *pActor, const WSTRING &strBuddyID, int nUserStatus, const WSTRING &strNick, int nIPTStatus, int nFlag)
	{
		return new CNotBuddyUserStatusNotifyCommand(pActor, strBuddyID, nUserStatus, strNick, nIPTStatus, nFlag);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();
		for (; it != observers.end(); ++it)
		{
			if( (*it)->OnGTMessengerActor_UpdateNotBuddyUserStatus(m_pActor, m_strBuddyID.c_str(), m_nUserStatus, m_strNick.c_str(), m_nIPTStatus, m_nFlag) == false )
				break;
		}

		//LeaveCriticalSection(pCS);
	}
	
	CMessengerActor *m_pActor;
	WSTRING m_strBuddyID;
	int m_nUserStatus;
	WSTRING m_strNick;
	int m_nIPTStatus;
	int m_nFlag;
};



///////////////////////////////////////////////////
// CLASS CBuddyActionNotifyCommand
///////////////////////////////////////////////////

class CBuddyActionNotifyCommand : public INotifyCommand
{
public:
	CBuddyActionNotifyCommand(CMessengerActor *pActor, const P_GT_BUDDY_RESULT_DATA &sResult) : INotifyCommand(pActor),
		m_pActor(pActor),
		m_sResult(sResult) { }

	static CBuddyActionNotifyCommand* Create(CMessengerActor *pActor, const P_GT_BUDDY_RESULT_DATA &sResult)
	{
		return new CBuddyActionNotifyCommand(pActor, sResult);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();

		GT_BUDDY_RESULT_DATA sData;
		m_sResult.Convert(sData);
	
		for (; it != observers.end(); ++it)
			if((*it)->OnGTMessengerActor_ResponseBuddyAction(m_pActor, sData) == false)
				break;

		//LeaveCriticalSection(pCS);
	}
	
	CMessengerActor *m_pActor;
	P_GT_BUDDY_RESULT_DATA m_sResult;
};




///////////////////////////////////////////////////
// CLASS CBuddyGroupActionNotifyCommand
///////////////////////////////////////////////////

class CBuddyGroupActionNotifyCommand : public INotifyCommand
{
public:
	CBuddyGroupActionNotifyCommand(CMessengerActor *pActor, const P_GT_BUDDY_GROUP_RESULT_DATA &sResult, GTBuddyAddGroupType addGroupType) : INotifyCommand(pActor),
		m_pActor(pActor),
		m_sResult(sResult),
		m_addGroupType(addGroupType){ }

	static CBuddyGroupActionNotifyCommand* Create(CMessengerActor *pActor, const P_GT_BUDDY_GROUP_RESULT_DATA &sResult, GTBuddyAddGroupType addGroupType)
	{
		return new CBuddyGroupActionNotifyCommand(pActor, sResult, addGroupType);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();

		GT_BUDDY_GROUP_RESULT_DATA sData;
		m_sResult.Convert(sData);

		for (; it != observers.end(); ++it)
			if((*it)->OnGTMessengerActor_ResponseBuddyGroupAction(m_pActor, sData, m_addGroupType) == false)
				break;

		//LeaveCriticalSection(pCS);
	}
	
	CMessengerActor *m_pActor;
	P_GT_BUDDY_GROUP_RESULT_DATA m_sResult;
	GTBuddyAddGroupType m_addGroupType;
};



///////////////////////////////////////////////////
// CLASS CBuddyGroupActionNotifyCommand
///////////////////////////////////////////////////

class CBuddyGroupSortOrderNotifyCommand : public INotifyCommand
{
public:
	CBuddyGroupSortOrderNotifyCommand(CMessengerActor *pActor, trid_t trid, vector<GT_BUDDY_GROUP_DATA> &vecBuddyGroupData) : INotifyCommand(pActor),
		m_pActor(pActor),
		m_trid(trid),
		m_vecBuddyGroupData(vecBuddyGroupData){ }

	static CBuddyGroupSortOrderNotifyCommand* Create(CMessengerActor *pActor, trid_t trid, vector<GT_BUDDY_GROUP_DATA> &vecBuddyGroupData)
	{
		return new CBuddyGroupSortOrderNotifyCommand(pActor, trid, vecBuddyGroupData);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();

		for (; it != observers.end(); ++it)
		{
			if((*it)->OnGTMessengerActor_ResponseUpdateGroupOrder(m_pActor, m_trid, m_vecBuddyGroupData) == false)
				break;
		}

		//LeaveCriticalSection(pCS);
	}

	CMessengerActor *m_pActor;
	
	trid_t m_trid;
	vector<GT_BUDDY_GROUP_DATA> &m_vecBuddyGroupData;
};



///////////////////////////////////////////////////
// CLASS CBuddyInviteNotifyCommand
///////////////////////////////////////////////////

class CBuddyInviteNotifyCommand : public INotifyCommand
{
public:
	CBuddyInviteNotifyCommand(CMessengerActor *pActor, const P_GT_BUDDY_INVITE_NOTIFY_DATA& sNotify) : INotifyCommand(pActor),
		m_pActor(pActor),
		m_sNotify(sNotify) { }

	static CBuddyInviteNotifyCommand* Create(CMessengerActor *pActor, const P_GT_BUDDY_INVITE_NOTIFY_DATA& sNotify)
	{
		return new CBuddyInviteNotifyCommand(pActor, sNotify);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();

		GT_BUDDY_INVITE_NOTIFY_DATA sData;
		m_sNotify.Convert(sData);

		for (; it != observers.end(); ++it)
			if((*it)->OnGTMessengerActor_NotifyInviteBuddy(m_pActor, sData) == false)
				break;

		//LeaveCriticalSection(pCS);
	}
	
	CMessengerActor *m_pActor;

	P_GT_BUDDY_INVITE_NOTIFY_DATA m_sNotify;
};


///////////////////////////////////////////////////////////////////////////////
// IPT
///////////////////////////////////////////////////////////////////////////////

class CIPTCallInfoNotifyCommand : public INotifyCommand
{
public:
	CIPTCallInfoNotifyCommand(CMessengerActor *pActor, LPCWSTR strUserID, LPCWSTR strTelNo, LPCWSTR strCID) 
		: INotifyCommand(pActor),
		m_pActor(pActor),
		m_strUserID(strUserID), 
		m_strTelNo(strTelNo), 
		m_strCID(strCID) { }

	static CIPTCallInfoNotifyCommand* Create(CMessengerActor *pActor, LPCWSTR strUserID, LPCWSTR strTelNo, LPCWSTR strCID)
	{
		return new CIPTCallInfoNotifyCommand(pActor, strUserID, strTelNo, strCID);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();

		for (; it != observers.end(); ++it)
		{
			if((*it)->OnGTMessengerActor_NotifyIPTCallInfo(m_pActor, m_strUserID, m_strTelNo, m_strCID) == false)
				break;
		}

		//LeaveCriticalSection(pCS);
	}

	CMessengerActor *m_pActor;

	LPCWSTR m_strUserID, m_strTelNo, m_strCID;
};

class CIPTResponsePhoneCallNotifyCommand : public INotifyCommand
{
public:
	CIPTResponsePhoneCallNotifyCommand(CMessengerActor *pActor, BYTE result, LPCWSTR strCID) 
		: INotifyCommand(pActor),
		m_pActor(pActor),
		m_bResult(result), 
		m_strCID(strCID) { }

	static CIPTResponsePhoneCallNotifyCommand* Create(CMessengerActor *pActor, BYTE result, LPCWSTR strCID)
	{
		return new CIPTResponsePhoneCallNotifyCommand(pActor, result, strCID);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();

		for (; it != observers.end(); ++it)
		{
			if((*it)->OnGTMessengerActor_ResponsePhoneCall(m_pActor, m_bResult, m_strCID) == false)
				break;
		}

		//LeaveCriticalSection(pCS);
	}

	CMessengerActor *m_pActor;

	BYTE m_bResult;
	LPCWSTR m_strCID;
};


class CIPTResponseForwardingInfoNotifyCommand : public INotifyCommand
{
public:
	CIPTResponseForwardingInfoNotifyCommand(CMessengerActor *pActor, BYTE ret, BYTE act, LPCWSTR strTelNo) 
		: INotifyCommand(pActor),
		m_pActor(pActor),
		m_ret(ret), 
		m_act(act),
		m_strTelNo(strTelNo) { }

	static CIPTResponseForwardingInfoNotifyCommand* Create(CMessengerActor *pActor, BYTE ret, BYTE act, LPCWSTR strTelNo)
	{
		return new CIPTResponseForwardingInfoNotifyCommand(pActor, ret, act, strTelNo);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();

		for (; it != observers.end(); ++it)
		{
			if((*it)->OnGTMessengerActor_ResponseForwardingInfo(m_pActor, m_ret, m_act, m_strTelNo) == false)
				break;
		}

		//LeaveCriticalSection(pCS);
	}

	CMessengerActor *m_pActor;

	BYTE m_ret;
	BYTE m_act;
	LPCWSTR m_strTelNo;
};

class CIPTResponseDndInfoNotifyCommand : public INotifyCommand
{
public:
	CIPTResponseDndInfoNotifyCommand(CMessengerActor *pActor, BYTE ret, BYTE act) 
		: INotifyCommand(pActor),
		m_pActor(pActor),
		m_ret(ret), 
		m_act(act) { }

	static CIPTResponseDndInfoNotifyCommand* Create(CMessengerActor *pActor, BYTE ret, BYTE act)
	{
		return new CIPTResponseDndInfoNotifyCommand(pActor, ret, act);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();

		for (; it != observers.end(); ++it)
		{
			if((*it)->OnGTMessengerActor_ResponseDndInfo(m_pActor, m_ret, m_act) == false)
				break;
		}

		//LeaveCriticalSection(pCS);
	}

	CMessengerActor *m_pActor;

	BYTE m_ret;
	BYTE m_act;
};

class CIPTResponseResultNotifyCommand : public INotifyCommand
{
public:
	CIPTResponseResultNotifyCommand(CMessengerActor *pActor, BYTE ret, uint16_t usIPTCMMD) 
		: INotifyCommand(pActor),
		m_pActor(pActor),
		m_ret(ret),
		m_usIPTCMMD(usIPTCMMD){ }

	static CIPTResponseResultNotifyCommand* Create(CMessengerActor *pActor, BYTE ret, uint16_t usIPTCMMD)
	{
		return new CIPTResponseResultNotifyCommand(pActor, ret, usIPTCMMD);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();

		for (; it != observers.end(); ++it)
		{
			if((*it)->OnGTMessengerActor_ResponseIptResult(m_pActor, m_ret, m_usIPTCMMD) == false)
				break;
		}

		//LeaveCriticalSection(pCS);
	}

	CMessengerActor *m_pActor;

	BYTE m_ret;
	uint16_t m_usIPTCMMD;
};





///////////////////////////////////////////////////
// CLASS CNewInstalledMobileUnknownUserNotifyCommand
///////////////////////////////////////////////////

class CNewInstalledMobileUnknownUserNotifyCommand : public INotifyCommand
{
public:
	CNewInstalledMobileUnknownUserNotifyCommand(CMessengerActor *pActor, int nCacheSerial) : INotifyCommand(pActor),
	m_pActor(pActor),
	m_nCacheSerial(nCacheSerial) { }
	
	static CNewInstalledMobileUnknownUserNotifyCommand* Create(CMessengerActor *pActor, int nCacheSerial)
	{
		return new CNewInstalledMobileUnknownUserNotifyCommand(pActor, nCacheSerial);
	}
	
	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);
		
		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();
		
		for (; it != observers.end(); ++it)
			if((*it)->OnGTMessengerActor_NotifyNewInstalledMobileUnknownUser(m_pActor, m_nCacheSerial) == false)
				break;

		//LeaveCriticalSection(pCS);
	}
	
	CMessengerActor *m_pActor;
	int m_nCacheSerial;
};





///////////////////////////////////////////////////
// CLASS CNewInstalledMobileUserNotifyCommand
///////////////////////////////////////////////////

class CNewInstalledMobileUserNotifyCommand : public INotifyCommand
{
public:
	CNewInstalledMobileUserNotifyCommand(CMessengerActor *pActor, LPCWSTR strUserID, LPCSTR strMobilePhoneHash) : INotifyCommand(pActor),
	m_pActor(pActor),
	m_strUserID(strUserID),
	m_strMobilePhoneHash(strMobilePhoneHash) { }
	
	static CNewInstalledMobileUserNotifyCommand* Create(CMessengerActor *pActor, LPCWSTR strUserID, LPCSTR strMobilePhoneHash)
	{
		return new CNewInstalledMobileUserNotifyCommand(pActor, strUserID, strMobilePhoneHash);
	}
	
	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();
		
		for (; it != observers.end(); ++it)
			if((*it)->OnGTMessengerActor_NotifyNewInstalledMobileUser(m_pActor, m_strUserID.c_str(), m_strMobilePhoneHash.c_str()) == false)
				break;

		//LeaveCriticalSection(pCS);
	}
	
	CMessengerActor *m_pActor;

	WSTRING m_strUserID;
	std::string m_strMobilePhoneHash;
};


///////////////////////////////////////////////////
// CLASS CBuddyUserStringNotifyCommand
///////////////////////////////////////////////////

class CBuddyUserStringNotifyCommand : public INotifyCommand
{
public:
	CBuddyUserStringNotifyCommand(CMessengerActor *pActor, unsigned int nUserDefinedCode, LPCWSTR strFromUserID, const std::string & strUserString) : INotifyCommand(pActor),
	m_pActor(pActor),
	m_nUserDefinedCode(nUserDefinedCode),
	m_strUserID(strFromUserID),
	m_strUserString(strUserString) { }
	
	static CBuddyUserStringNotifyCommand* Create(CMessengerActor *pActor, unsigned int nUserDefinedCode, LPCWSTR strFromUserID, const std::string & strUserString)
	{
		return new CBuddyUserStringNotifyCommand(pActor, nUserDefinedCode, strFromUserID, strUserString);
	}
	
	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();
		
		for (; it != observers.end(); ++it)
			if((*it)->OnGTMessengerActor_NotifyRecvUserString(m_pActor, m_nUserDefinedCode, m_strUserID.c_str(), m_strUserString.c_str()) == false)
				break;

		//LeaveCriticalSection(pCS);
	}
	
	CMessengerActor *m_pActor;
	int m_nUserDefinedCode;
	WSTRING m_strUserID;
	std::string m_strUserString;
};



///////////////////////////////////////////////////
// CLASS CListBuddyNotifyCommand
///////////////////////////////////////////////////

class CListBuddyNotifyCommand : public INotifyCommand
{
public:
	CListBuddyNotifyCommand(CMessengerActor *pActor) : INotifyCommand(pActor),
		m_pActor(pActor) { }

	static CListBuddyNotifyCommand* Create(CMessengerActor *pActor)
	{
		return new CListBuddyNotifyCommand(pActor);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();
		for (; it != observers.end(); ++it)
			if((*it)->OnGTMessengerActor_UpdateListBuddy(m_pActor) == false)
				break;

		//LeaveCriticalSection(pCS);
	}
	
	CMessengerActor *m_pActor;
};




///////////////////////////////////////////////////
// CLASS CMMCInviteNotifyCommand
///////////////////////////////////////////////////

class CMMCInviteNotifyCommand : public INotifyCommand
{
public:
	CMMCInviteNotifyCommand(CMessengerActor *pActor, const P_GT_ROOM_INVITE_NOTIFY_DATA& sNotify) : INotifyCommand(pActor),
		m_pActor(pActor),
		m_sNotify(sNotify) { }

	static CMMCInviteNotifyCommand* Create(CMessengerActor *pActor, const P_GT_ROOM_INVITE_NOTIFY_DATA& sNotify)
	{
		return new CMMCInviteNotifyCommand(pActor, sNotify);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();

		GT_ROOM_INVITE_NOTIFY_DATA sData;
		m_sNotify.Convert(sData);

		for (; it != observers.end(); ++it)
			if((*it)->OnGTMessengerActor_NotifyInviteRoom(m_pActor, sData) == false)
				break;

		//LeaveCriticalSection(pCS);
	}
	
	CMessengerActor *m_pActor;

	P_GT_ROOM_INVITE_NOTIFY_DATA m_sNotify;
};

///////////////////////////////////////////////////
// CLASS CSetMyProfileImageNotifyCommand
///////////////////////////////////////////////////

class CSetMyProfileImageNotifyCommand : public INotifyCommand
{
public:
	CSetMyProfileImageNotifyCommand(CProfileImageActor *pActor,
									const P_GT_PROFILE_IMAGE_RESULT_DATA & sData) : INotifyCommand(pActor),
		m_pActor(pActor),
		m_sData(sData) { }

	static CSetMyProfileImageNotifyCommand* Create(CProfileImageActor *pActor, 
												   const P_GT_PROFILE_IMAGE_RESULT_DATA & sData)
	{
		return new CSetMyProfileImageNotifyCommand(pActor, sData);
	}

	virtual void Doit(void)
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTProfileImageActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTProfileImageActorEvent>::listobserver_t::iterator it = observers.begin();

		GT_PROFILE_IMAGE_RESULT_DATA sData;
		m_sData.Convert(sData);

		for (; it != observers.end(); ++it)
			if((*it)->OnGTProfileImageActor_ResponseSetMyProfileImage(m_pActor, sData) == false)
				break;

		//LeaveCriticalSection(pCS);
	}

	CProfileImageActor *m_pActor;
	P_GT_PROFILE_IMAGE_RESULT_DATA m_sData;
};


///////////////////////////////////////////////////
// CLASS CProfileImageResponseNotifyCommand
///////////////////////////////////////////////////

class CProfileImageResponseNotifyCommand : public INotifyCommand
{
public:
	CProfileImageResponseNotifyCommand(CProfileImageActor *pActor,
                                       const P_GT_PROFILE_IMAGE_RESULT_DATA & sResult,
                                       DWORD dwUserData1,
                                       DWORD dwUserData2,
                                       bool bCachedImage) : INotifyCommand(pActor),
    m_pActor(pActor),
    m_sResult(sResult),
    m_dwUserData1(dwUserData1),
    m_dwUserData2(dwUserData2),
    m_bCachedImage(bCachedImage) { }
    
	static CProfileImageResponseNotifyCommand* Create(CProfileImageActor *pActor, 
                                                      const P_GT_PROFILE_IMAGE_RESULT_DATA & sResult,
                                                      DWORD dwUserData1,
                                                      DWORD dwUserData2,
                                                      bool bCachedImage)
	{
		return new CProfileImageResponseNotifyCommand(pActor, sResult, dwUserData1, dwUserData2, bCachedImage);
	}

	virtual void Doit(void)
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTProfileImageActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTProfileImageActorEvent>::listobserver_t::iterator it = observers.begin();

		GT_PROFILE_IMAGE_RESULT_DATA sData;
		m_sResult.Convert(sData);

		for (; it != observers.end(); ++it)
			if((*it)->OnGTProfileImageActor_ResponseProfileImage(m_pActor, sData, m_dwUserData1, m_dwUserData2, m_bCachedImage) == false)
				break;

		//LeaveCriticalSection(pCS);
	}

	CProfileImageActor *m_pActor;
	P_GT_PROFILE_IMAGE_RESULT_DATA m_sResult;
    DWORD m_dwUserData1;
    DWORD m_dwUserData2;
    bool m_bCachedImage;
};


///////////////////////////////////////////////////
// CLASS CBuddyRefreshWorkStatusNotifyCommand
///////////////////////////////////////////////////

class CBuddyRefreshWorkStatusNotifyCommand : public INotifyCommand
{
public:
	CBuddyRefreshWorkStatusNotifyCommand(CMessengerActor *pActor, trid_t trid, vector<GT_BUDDY_WORK_STATUS_DATA> &vecBuddyWorkStatusData) : INotifyCommand(pActor),
		m_pActor(pActor),
		m_trid(trid),
		m_pvecBuddyWorkStatusData(vecBuddyWorkStatusData){ }

	static CBuddyRefreshWorkStatusNotifyCommand* Create(CMessengerActor *pActor, trid_t trid, vector<GT_BUDDY_WORK_STATUS_DATA> &vecBuddyWorkStatusData)
	{
		return new CBuddyRefreshWorkStatusNotifyCommand(pActor, trid, vecBuddyWorkStatusData);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();

		for (; it != observers.end(); ++it)
		{
			if((*it)->OnGTMessengerActor_ResponseRefreshWorkStatus(m_pActor, m_trid, m_pvecBuddyWorkStatusData) == false)
				break;
		}
	}

	CMessengerActor *m_pActor;

	trid_t m_trid;
	vector<GT_BUDDY_WORK_STATUS_DATA> &m_pvecBuddyWorkStatusData;
};

class CUserImageNotifyCommand : public INotifyCommand
{
public:
	CUserImageNotifyCommand(CMessengerActor *pActor, const vector<P_GT_USER_IMAGE_RESULT_DATA> &sResult) : INotifyCommand(pActor),
		m_pActor(pActor),
		m_sResult(sResult) { }

	static CUserImageNotifyCommand* Create(CMessengerActor *pActor, vector<P_GT_USER_IMAGE_RESULT_DATA> &sResult)
	{
		return new CUserImageNotifyCommand(pActor, sResult);
	}

	virtual void Doit(void) 
	{
		GT_ASSERT(m_pActor);

		//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
		//EnterCriticalSection(pCS);

		CGTObserved<IGTMessengerActorEvent>::listobserver_t observers;
		m_pActor->GetObservers(&observers);
		CGTObserved<IGTMessengerActorEvent>::listobserver_t::iterator it = observers.begin();

		vector<GT_USER_IMAGE_RESULT_DATA> sData;
		for(int nIndex = 0; nIndex < (int)m_sResult.size(); nIndex++)
		{
			GT_USER_IMAGE_RESULT_DATA data;
			m_sResult[nIndex].Convert(data);
			sData.push_back(data);
		}
		//m_sResult.Convert(sData);

		for (; it != observers.end(); ++it)
			if((*it)->OnGTMessengerActor_ResponseUserImage(m_pActor, sData) == false)
				break;

		//LeaveCriticalSection(pCS);
	}

	CMessengerActor *m_pActor;
	vector<P_GT_USER_IMAGE_RESULT_DATA> m_sResult;
};
